@@ -115,6 +115,49 @@ class ListAsDict(list):
 
 track_current_drop = object_tracking("drop")
 
+class InstanceLogHandler(logging.Handler):
+    """Custom handler to store logs in-memory per object instance."""
+    def __init__(self, log_storage):
+        super().__init__()
+        self.log_storage = log_storage
+
+    def emit(self, record):
+        """Store log messages in the instance's log storage.
+
+         :param record: The log string we want to add to the log storage
+
+         .. note: We are not interested in actually emitting the log;
+             we are just interested in extracting and storing Record metadata
+        """
+
+        exc = f"{str(record.exc_text)}" if record and record.exc_text else ""
+        # msg = str(record.message).replace("\n", "<br>")
+        msg = (f"<pre>{record.message.encode('utf-8').decode('unicode_escape')}\n"
+               f"{exc}</pre>")
+        try:
+            rec_time = record.asctime
+        except AttributeError:
+            rec_time = ""
+        self.log_storage.append({ "time":rec_time,
+            "Level": record.levelname,
+            "Module": record.name,
+            "Function/Method": record.funcName,
+            "Line #": record.lineno,
+            "Message": msg,
+        })
+
+
+class DROPLogFilter(logging.Filter):
+    def __init__(self, uid: str, humanKey: str):
+        super().__init__()
+        self.uid = uid
+        self.humanKey = humanKey
+
+    def filter(self, record):
+        uid = getattr(record, "drop_uid", None)
+        return uid in [self.uid, self.humanKey]
+
+
 
 # ===============================================================================
 # DROP classes follow
@@ -226,6 +269,26 @@ class AbstractDROP(EventFirer, EventHandler):
             self._log_level = logging.getLevelName(logger.getEffectiveLevel())
         self._global_log_level = logging.getLevelName(logger.getEffectiveLevel())
 
+        self._humanKey = self._popArg(
+            kwargs, "humanReadableKey", truncateUidToKey(self._uid)
+        )
+
+        self.log_storage = []
+
+        self.logger = logging.getLogger(f"{__class__}.{self.uid}")
+        instance_handler = InstanceLogHandler(self.log_storage)
+        instance_handler.addFilter(DROPLogFilter(self.uid, self._humanKey))
+        fmt = ("%(asctime)-15s [%(levelname)5.5s] "
+               "%(name)s#%(funcName)s:%(lineno)s %(message)s")
+        fmt = logging.Formatter(fmt)
+        instance_handler.setFormatter(fmt)
+
+        # Attach instance-specific handler
+        logging.root.addHandler(instance_handler)
+
+        # Ensure logs still propagate to the root logger
+        logger.propagate = True
+
         # The physical graph drop type. This is determined
         # by the drop category when generating the drop spec
         self._type = self._popArg(kwargs, "categoryType", None)
@@ -325,9 +388,6 @@ class AbstractDROP(EventFirer, EventHandler):
         # TODO: Make these threadsafe, no lock around them yet
         self._rios = {}
 
-        self._humanKey = self._popArg(
-            kwargs, "humanReadableKey", truncateUidToKey(self._uid)
-        )
         # The execution mode.
         # When set to DROP (the default) the graph execution will be driven by
         # DROPs themselves by firing and listening to events, and reacting
@@ -433,7 +493,7 @@ class AbstractDROP(EventFirer, EventHandler):
                     "InputOutput",
                     ]
                 ):
-                    return kwargs["applicationArgs"].get(attr_name).value
+                return kwargs["applicationArgs"].get(attr_name).value
             else:
                 return default_value
 
