@@ -2,12 +2,12 @@ import ast
 import binascii
 import logging
 import collections
+import pickle
 
 from dlg.ddap_protocol import DROPStates
 from dlg.utils import deserialize_data
 import numpy as np
 
-from dlg.data.drops.data_base import DataDROP
 import dlg.droputils as droputils
 import dlg.drop_loaders as drop_loaders
 
@@ -55,7 +55,7 @@ class Argument:
     """
 
     value: object
-    encoding: DropParser = DropParser.DILL
+    encoding: DropParser = None #DropParser.DILL
     type: Union [ArgType, None] = None
     precious: bool = False
     positional: bool = False
@@ -183,11 +183,11 @@ def identify_named_ports(
                 value = positionalPortArgs[key].value
                 logger.warning("Input drop skipped! Using %s default value for parg %s", mode, key)
             elif local_parser:
-                logger.debug("Reading from %s encoded port %s using %s", encoding, key, parser.__repr__())
+                logger.debug("Reading from %s encoded port %s using %s", encoding, key, local_parser.__repr__())
                 value = local_parser(port_dict[keys[i]]["drop"])
                 try:
                     value = deserialize_data(value)
-                except Exception:
+                except (TypeError, AttributeError, binascii.Error, pickle.UnpicklingError):
                     # If deserialization does not work we just
                     # stick with the value
                     pass
@@ -466,12 +466,6 @@ def get_port_reader_function(input_parser: DropParser):
     # Inputs are un-pickled and treated as the arguments of the function
     # Their order must be preserved, so we use an OrderedDict
     ip = None
-    # if isinstance(input_parser, str):
-    #     parsers = DropParser.__members__
-    #     ip = input_parser.upper()
-    #     ip = parsers[ip] if ip in parsers else None
-    # else:
-    #     ip = input_parser
     ip = resolve_drop_parser(input_parser)
     if ip is DropParser.PICKLE:
         # all_contents = lambda x: pickle.loads(droputils.allDropContents(x))
@@ -488,17 +482,7 @@ def get_port_reader_function(input_parser: DropParser):
             return ast.literal_eval(content) if len(content) > 0 else None
 
         reader = optionalEval
-    elif ip is DropParser.UTF8:
-
-        def utf8decode(drop: "DataDROP"):
-            """
-            Decode utf8
-            Not stored in drop_loaders to avoid cyclic imports
-            """
-            return droputils.allDropContents(drop).decode("utf-8")
-
-        reader = utf8decode
-    elif ip is DropParser.NPY:
+    elif input_parser is DropParser.NPY:
         reader = drop_loaders.load_npy
     elif ip is DropParser.PATH:
         def PathFromData(x: AbstractDROP):
